<?php
namespace AngelicaO;
use MNC\MNC_API;


class Locations {
    public function __construct() {
    }


    /**
     * Gets all locations with their children locations, recursively. Sorted alpha.ly
     * @return array
     * @throws \Exception
     */
    public static function getAllLocations() {
        $locations = array();

        $sql = 'SELECT * FROM locations';
        $db_locations = DB::query($sql, array());

        if (empty($parent_id)) {
            foreach ($db_locations as $location) {
                if (empty($location['parent_id'])) {
                    $location['children'] = self::getLocationChildren($location['id'], $db_locations);
                    $locations[] = $location;

                }
            }
        } else {
            return self::getLocationChildren(intval($parent_id), $db_locations);
        }

        return $locations;
    }

    /**
     * Gets all locations with their children locations, recursively. Sorted alpha.ly
     * @param null $parent_id
     * @return array
     * @throws \Exception
     */
    public static function getLocations($parent_id = null) {
        $locations = array();

        $sql = 'SELECT * FROM locations';
        $db_locations = DB::query($sql, array());

        if (empty($parent_id)) {
            foreach ($db_locations as $location) {
                if (empty($location['parent_id'])) {//we dont want sublocations at level 1 of the array
                    $location['children'] = self::getLocationChildren($location['id'], $db_locations);
                    if (!empty($location['children'])) {
                        $location['has_children'] = true;
                    }
                    $locations[] = $location;

                }
            }
        } else {
            return self::getLocationChildren(intval($parent_id), $db_locations);
        }

        return $locations;

    }


    public static function getLocation($id) {
        if (empty($id) && $id !== 0) {
            return array(
                'status' => 'error',
                'message' => 'ID cannot be empty'
            );
        }

        $location = null;

        $data = array(
            'id' => $id
        );
        $sql = 'SELECT * FROM locations WHERE id = ?:[id,i] LIMIT 1';
        $location = DB::query($sql, $data);

        if (empty($location)) {
            return array(
                'status' => 'error',
                'message' => 'unable to get location'
            );
        } else {
            return reset($location);
        }

    }

    /**
     * Recursive function to get all children of a location.
     * @param $parent_id Id of the parent
     * @param $location_list list of all location IDs
     * @return mixed Array containing children [id, name, children => [...] ]
     */
    private static function getLocationChildren($parent_id, $location_list) {
        $children = array();
        foreach ($location_list as $location) {
            if (!empty($location['parent_id']) && intval($location['parent_id']) === intval($parent_id)) {
                $location['children'] = self::getLocationChildren($location['id'], $location_list);
                if (!empty($location['children'])) {
                    $location['has_children'] = true;
                }
                $children[] = $location;
            }
        }
        return $children;
    }

    public static function addLocation($location) {
        if (empty($location['name'])) {
            return array(
                'status' => 'error',
                'message' => 'Name cannot be empty'
            );
        }

        $data = array(
            'name' => $location['name'],
        );
        if (!empty($location['parent_id'])) {
            $data['parent_id'] = $location['parent_id'];
        }
        $sql = 'INSERT INTO locations SET name = ?:[name,s] ' . (!empty($data['parent_id']) ? ' , parent_id = ?:[parent_id,i] ' : '');
        $location_id = DB::query($sql, $data, array('return_insert_id' => true));

        if (empty($location_id) && $location_id !== 0) {
            return array(
                'status' => 'error',
                'message' => 'Error creating location'
            );
        }

        return array(
            'name' => $location['name'],
            'id' => $location_id,
            'parent_id' => (!empty($location['parent_id']) ? intval($location['parent_id']) : '')
        );
    }

    public static function deleteLocation($id) {

        if (empty($id) && $id !== 0) {
            return array(
                'status' => 'error',
                'message' => 'ID cannot be empty'
            );
        }
        $id = intval($id);

        //check if there are children to this location
        $all_locations = self::getLocations();
        $children = array();
        foreach ($all_locations as $location) {
            if (intval($location['parent_id']) === $id) {
                $children[] = $location;
            }
        }

        if (sizeof($children) > 0) {
            //if there are children, then call this function again to
            foreach ($children as $child) {
                self::deleteLocation($child['id']);
            }
        }

        $data = array(
            'id' => $id
        );

        //remove any references to this location among devices
        $sql = 'UPDATE devices SET location_id = NULL WHERE location_id = ?:[id,i]';
        DB::query($sql, $data);

        $sql = 'DELETE FROM locations WHERE id = ?:[id,i] LIMIT 1';
        DB::query($sql, $data);

        return array(
            'status' => 'success'
        );
    }

    /**
     * DEPRECATED
     * @param $locations
     * @param null $parent
     * @return array
     * @throws \Exception
     */
    public static function moveLocations($locations, $parent = null) {
        if ($parent === null || !isset($parent['id'])) {
            $parent = null;
        } else {
            $parent = $parent['id'];
        }

        $subject_ids = array();
        foreach ($locations as $location) {
            if (intval($location['id']) === intval($parent)) {
                continue;//since we can't move a location to itself :)
            }
            $subject_ids[] = intval($location['id']);
        }

        $data = array(
            'parent_id' => $parent,
            'subjects' => $subject_ids
        );

        $sql = 'UPDATE locations SET parent_id = ' . ($parent !== null ? '?:[parent_id,i]' : 'null') . ' WHERE id IN ?:[subjects,z]';
        DB::query($sql, $data);

        return array(
            'status' => 'success'
        );
    }

    public static function moveChildrenLocations($fromParent, $toParent = null) {
        $fromParentID = null;
        $toParentID = null;
        if (!empty($fromParent) && isset($fromParent['id']) && is_numeric($fromParent['id'])) {
            $fromParentID = intval($fromParent['id']);
        }

        if (!empty($toParent) && isset($toParent['id']) && is_numeric($toParent['id'])) {
            $toParentID = intval($toParent['id']);
        }

        $data = array(
            'fromParentID' => $fromParentID,
            'toParentID' => $toParentID
        );

        $sql = 'UPDATE locations SET parent_id = ' . ($toParentID !== null ? '?:[toParentID,i]' : 'null') . ' WHERE parent_id = ?:[fromParentID,i]';
        DB::query($sql, $data);

        return array(
            'status' => 'success'
        );
    }

    public static function renameLocation($location_id, $newName) {
        $data = array(
            'id' => $location_id,
            'newName' => $newName
        );

        $sql = 'UPDATE locations SET name = ?:[newName,s] WHERE id = ?:[id,i]';
        DB::query($sql, $data);

        return array(
            'status' => 'success'
        );
    }

    public static function setPalette($location_id, $palette) {
        if (empty($palette['bg']) || empty($palette['color'])) {
            return array(
                'status' => 'error',
                'message' => 'Palette bg or color cannot be empty'
            );
        }
        $data = array(
            'id' => $location_id,
            'palette_bg' => $palette['bg'],
            'palette_color' => $palette['color']
        );

        $sql = 'UPDATE locations SET palette_bg = ?:[palette_bg,s], palette_color = ?:[palette_color,s] WHERE id = ?:[id,i]';
        DB::query($sql, $data);

        return array(
            'status' => 'success'
        );
    }

    public static function setVisualMode($location) {
        if (empty($location['background_image'])) {

            if (empty($location['visual_mode'])) {
                //empty out the visual mode data

                $sql = 'UPDATE locations SET background_image = NULL, visual_mode = NULL, stage_width = NULL, stage_height = NULL WHERE id = ?:[id,i]';

                DB::query($sql, array('id' => $location['id']));

                return array(
                    'status' => 'success'
                );
            } else if ($location['visual_mode'] === 'blank') {
                return self::setVisualModeBlank(array('id' => $location['id']));
            }

        } else {

            return self::setVisualModeBackgroundImage(array(
                'id' => $location['id'],
                'background_image' => $location['background_image']
            ));
        }

    }

    private static function setVisualModeBlank($location) {

        $sql = 'UPDATE locations SET background_image = NULL, visual_mode = "blank" WHERE id = ?:[id,i]';

        DB::query($sql, array('id' => $location['id']));

        return array(
            'status' => 'success'
        );
    }


    private static function setVisualModeBackgroundImage($location) {

        $status = true;

        if (!empty($location) && !empty($location['background_image'])) {
            $bgImage = $location['background_image'];
            if (!is_dir('./uploads')) {
                if (!mkdir('uploads')) {
                    $status = false;
                }
            }

            if ($status) {
                $timestamp = time();
                //TODO sanitize the bgImage
                $image_url = 'uploads/' . $timestamp . '_' . intval($location['id']) .'.jpg';
                $file_saved = file_put_contents($image_url, base64_decode($bgImage));
                if ($file_saved === false) {
                    $status = false;
                }

                if ($status) {
                    $data = array(
                        'id' => $location['id'],
                        'image_path' => $image_url
                    );

                    $sql = 'UPDATE locations SET background_image = ?:[image_path,s], visual_mode = "bg" WHERE id = ?:[id,i]';
                    DB::query($sql, $data);
                }

            }
        }

        if (empty($status)) {
            return array(
                'status' => 'error',
                'message' => 'Error creating visual mode for location' . $location['id']
            );
        } else {
            return array(
                'status' => 'success'
            );
        }

    }


    public static function addDevice($id, $mac) {
        if (empty($id)) {
            return array(
                'status' => 'error',
                'message' => 'Location cannot be empty'
            );
        }

        if (empty($mac)) {
            return array(
                'status' => 'error',
                'message' => 'Device cannot be empty'
            );
        }

        $data = array(
            'id' => $id,
            'mac' => $mac
        );

        $sql = 'UPDATE devices SET location_id = ?:[id,i], translate_x = "", translate_y = "", isUsedInViewMode = "", fontSize = "" WHERE mac = ?:[mac,s] ';
        DB::query($sql, $data);


        return array(
            'status' => 'success'
        );
    }

    public static function addDevices($id, $devices) {
        if (empty($id)) {
            return array(
                'status' => 'error',
                'message' => 'Location cannot be empty'
            );
        }

        if (empty($devices)) {
            return array(
                'status' => 'error',
                'message' => 'Devices cannot be empty'
            );
        }


        $macs = array();
        foreach ($devices as $device) {
            $macs[] = $device['mac'];
        }

        if (!empty($macs)) {
            $data = array(
                'id' => $id,
                'macs' => $macs
            );

            $sql = 'UPDATE devices SET location_id = ?:[id,i], translate_x = "", translate_y = "", isUsedInViewMode = "", fontSize = "" WHERE mac IN ?:[macs,x] ';
            DB::query($sql, $data);
        }



        return array(
            'status' => 'success'
        );

    }

    public static function getDevices($id) {
        $MNCDevices = MNC_API::getDevices();

        if (empty($id)) {
            return array(
                'status' => 'error',
                'message' => 'Location cannot be empty'
            );
        }

        $user_account = MNC_API::getAccount();
        $data = array(
            'id' => $id,
            'user_id' => intval($user_account['user_id'])
        );

        $sql = 'SELECT devices.*, video_wall_layouts.name as `connectedLayout`,' .
            '(SELECT COUNT(*) FROM video_wall_currently_playing_connections WHERE videoWallFullConfigId = devices.videoWallFullConfigId) as `connectedLayoutChanges` FROM devices ' .
            'LEFT JOIN video_wall_layouts ON (devices.selected_layout = video_wall_layouts.layout_id)' .
            'WHERE mac NOT IN (SELECT mac FROM video_wall_config_devices UNION select mac_rx as `mac` from multiview_displays) AND location_id = ?:[id,i] AND deleted = 0 AND mac NOT IN (select mac from blocked_devices WHERE user_id = ?:[user_id,i])';

        $result_devices = DB::query($sql, $data);

        //merge the data from the MNC into the resulting array
        for ($i = 0; $i < sizeof($result_devices); $i++) {
            for ($j = 0; $j < sizeof($MNCDevices['simpleDevices']); $j++) {
                if ($result_devices[$i]['mac'] == $MNCDevices['simpleDevices'][$j]['mac']) {
                    //merge the data from the MNC into the default data (like connectedMac and other technical details with aren't stored in this database)
                    $result_devices[$i] = $result_devices[$i] + $MNCDevices['simpleDevices'][$j];//plus operator will not override items on the left operand
                    break;
                }
            }
            //TODO remove this once it's fixed; correct invalid model#s for some devices
            $result_devices[$i]['modelName'] = Devices::fixMalformattedModelName($result_devices[$i]['modelName']);
        }

        //set the activeSource data (what source is currently connected to the RX)
        Devices::buildActiveSourceData($result_devices);

        if (!empty($result_devices)) {
            return $result_devices;
        } else {
            return array(
            );
        }


    }

    public static function saveVisualModeData($id, $data) {
        if (empty($id)) {
            return array(
                'status' => 'error',
                'message' => 'Location cannot be empty'
            );
        }

        if (empty($data['devices'])) {
            return array(
                'status' => 'error',
                'message' => 'Devices cannot be empty'
            );
        }

       self::saveVisualModeDeviceData($id, $data['devices']);

        return array(
            'status' => 'success'
        );
    }

    private static function saveVisualModeDeviceData($id, $devices) {
        if (empty($id)) {
            return array(
                'status' => 'error',
                'message' => 'Location cannot be empty'
            );
        }

        if (empty($devices)) {
            return array(
                'status' => 'error',
                'message' => 'Devices cannot be empty'
            );
        }

        foreach ($devices as $device) {

            $data = array(
                'mac' => $device['mac'],
                'isUsedInViewMode' => (empty($device['isUsedInViewMode']) ? 0 : 1),
                'translate_x' => (!empty($device['position']['x']) ? $device['position']['x'] : 0),
                'translate_y' => (!empty($device['position']['y']) ? $device['position']['y'] : 0),
                'fontSize' => (!empty($device['fontSize']) ? $device['fontSize'] : '')
            );

            $sql = 'UPDATE devices SET isUsedInViewMode = ?:[isUsedInViewMode,i], translate_x = ?:[translate_x,s], translate_y = ?:[translate_y,s], fontSize = ?:[fontSize,s]  WHERE mac = ?:[mac,s] ';
            DB::query($sql, $data);
        }


        return array(
            'status' => 'success'
        );
    }

    public static function getPreferredSources($location_id, $filter_system_id) {
        if (emptynz($location_id)) {
            return array(
                'status' => 'error',
                'message' => 'location cannot be empty'
            );
        }

        $user_account = MNC_API::getAccount();

        //check if there are preferred sources for that location
        $data = array(
            'location_id' => $location_id,
            'user_id' => intval($user_account['user_id'])
        );
        $sql = 'SELECT source_mac, devices.* FROM location_preferred_sources LEFT JOIN devices ON devices.mac = location_preferred_sources.source_mac ';

        if (!empty($filter_system_id)) {
            $data['system_id'] = $filter_system_id;
            $sql .= ' AND devices.mac = location_preferred_sources.source_mac ';//TODO huh? shouldn't this say devices.system_id = ?:[system_id,i] ???
        }

        $sql .= ' WHERE location_preferred_sources.location_id = ?:[location_id,i] AND devices.isDisconnected != 1 ';

        if (!empty($filter_system_id)) {
            $sql .= ' AND system_id = ?:[system_id,i] ';
        }

        $sql .= ' AND source_mac NOT IN (select mac from blocked_devices where user_id = ?:[user_id,i]) ';
        $preferred_sources = DB::query($sql, $data);

        return $preferred_sources;
    }

    public static function setPreferredSources($location_id, $sources) {
        if (emptynz($location_id)) {
            return array(
                'status' => 'error',
                'message' => 'location cannot be empty'
            );
        }

        $data = array(
            'location_id' => $location_id
        );

        $sql = 'DELETE FROM location_preferred_sources WHERE location_id = ?:[location_id,i]';
        DB::query($sql, $data);

        if (!empty($sources)) {
            $sql = 'REPLACE INTO location_preferred_sources (location_id, source_mac) VALUES ';

            for ($i = 0; $i < sizeof($sources); $i++) {
                if (!empty($sources[$i]['mac'])) {
                    if ($i > 0) {
                        $sql .= ', ';
                    }
                    $data['source_mac' . $i] = $sources[$i]['mac'];
                    $sql .= '(?:[location_id,i], ?:[source_mac' . $i . ',s])';
                }
            }

            DB::query($sql, $data);
        }

        return array(
            'status' => 'success'
        );
    }

    public static function getSourcesAvailable($location_id) {
        if (emptynz($location_id)) {
            return array(
                'status' => 'error',
                'message' => 'location must not be empty'
            );
        }

        $sources = array();

        $location_system_ids = array();

        $user_account = MNC_API::getAccount();
        $data = array(
            'location_id' => $location_id,
            'user_id' => intval($user_account['user_id'])
        );

        $sql = 'SELECT * FROM devices WHERE location_id = ?:[location_id,i] AND mac NOT IN (select mac from blocked_devices WHERE user_id = ?:[user_id,i])';//this will exclude TXs since they have no location_id
        $result_devices = DB::query($sql, $data);
        
        foreach ($result_devices as $result_device) {
            if (!in_array($result_device['system_id'], $location_system_ids)) {
                $location_system_ids[] = $result_device['system_id'];
            }
        }

        foreach ($location_system_ids as $location_system_id) {
            $sources[$location_system_id]['preferred_sources'] = self::getPreferredSources($location_id, $location_system_id);
            $sources[$location_system_id]['sources_in_system'] = Devices::getSourcesInSystem($location_system_id);
        }

        return $sources;
    }


    public static function getNavigation($location_id) {
        if (emptynz($location_id)) {
            return array(
                'status' => 'error',
                'message' => 'Location cannot be empty'
            );
        }

        $data = array(
            'location_id' => $location_id
        );

        $sql = 'SELECT l1.* FROM locations `l1` WHERE l1.parent_id <=> (SELECT l2.parent_id FROM locations `l2` WHERE l2.id = ?:[location_id,i]) AND NOT EXISTS (SELECT null FROM locations `l3` WHERE l3.parent_id = l1.id) ORDER BY name ASC';
        $locations = DB::query($sql, $data);

        return $locations;

    }



    /**
     *
     *
     * Utils
     *
     *
     */





    /**
     * Recursive function to find a location in the list of locations and their children.
     * @param $id
     * @param $locations
     * @return null
     */
    public static function utilFindLocationByID($id, $locations) {
        if (!isset($locations)) {
            return null;
        }

        for ($i = 0; $i < sizeof($locations); $i++) {
            if (intval($locations[$i]['id']) === intval($id)) {
                return $locations[$i];
            }

            if (!empty($locations[$i]['children'])) {
                $foundInChild = self::utilFindLocationByID($id, $locations[$i]['children']);
                if ($foundInChild !== null) {
                    return $foundInChild;
                }
            }
        }

        return null;
    }

    /**
     * Recursive function to build the path of a location. Returns an array of the locations that form the path, from most specific to most general.
     * @param $location
     * @param $locations
     * @return array
     */
    public static function utilGetLocationPath($location, $locations) {
        $path = array();

        if (isset($location['parent_id']) && is_numeric($location['parent_id'])) {
            $parent = self::utilFindLocationByID($location['parent_id'], $locations);

            if ($parent !== null) {
                $path[] = $parent['name'];
                if (isset($parent['parent_id']) && is_numeric($parent['parent_id'])) {
                    $parent_path = self::utilGetLocationPath($parent, $locations);
                    $path = array_merge($path, $parent_path);
                }
            }
        }

        return $path;
    }
}
