<?php
namespace AngelicaO;
use MNC\MNC_API;

class MultiView {

    public function __construct() {
    }


    /**
     *
     * When reloading the data from the MNC:
     * - Check if any RX changed (for now this isn't relevant, it will appear as a new multiview and
     * the old one as disconnected)
     * - Add / update the configs themselves
     * - Add / update the windows belonging to these configs
     */

    /**
     * @param $mnc_multiviews
     * @throws \Exception
     */
    public static function processMultiviewPayloadFromMNC($mnc_multiviews) {

        if (empty($mnc_multiviews)) {
            $mnc_multiviews = array();
        }

        $all_multiviews = self::getMultiviews();

        //clear the video wall subconfig and devices data for reinsertion right after
        DB::query('TRUNCATE TABLE multiview_displays', array());
        DB::query('TRUNCATE TABLE multiview_configs', array());
        DB::query('TRUNCATE TABLE multiview_config_windows', array());

        if (!empty($mnc_multiviews)) {
            foreach ($mnc_multiviews as $mnc_multiview) {
                //check if the device exists by main config id (meaning the config id of the "main" config. i.e the config that has all TVs assigned
                $exists = false;
                for ($i = 0; $i < sizeof($all_multiviews); $i++) {
                    //TODO temp / fake code I don't know the key of the MV ID
                    if ($mnc_multiview['system_id'] . '_mv_' . strtolower($mnc_multiview['multiview_id']) == strtolower($all_multiviews[$i]['mac'])) {
                        $exists = $i;
                    }
                }
                if ($exists === false) {
                    self::addMultiview(array(
                        //TODO field names fake / temp
                        'customName' => $mnc_multiview['VideoWallName'],
                        'system_id' => $mnc_multiview['system_id'],
                        'mac' => $mnc_multiview['system_id'] . '_mv_' . strtolower($mnc_multiview['multiview_id']),
                        'videoWallSize' => $mnc_multiview['vwSize'],
                        'configs' => (isset($mnc_multiview['configs']) ? $mnc_multiview['configs'] : array()),
                        'displays' => (isset($mnc_multiview['devices']) ? $mnc_multiview['displays'] : array())
                    ));

                } else {

                    self::updateMultiview(array(
                        //todo fake / temp field names
                        'customName' => $mnc_multiview['VideoWallName'],
                        'system_id' => $mnc_multiview['system_id'],
                        'mac' => $mnc_multiview['system_id'] . '_mv_' . strtolower($mnc_multiview['multiview_id']),
                        'videoWallSize' => $mnc_multiview['vwSize'],
                    ));
                    //re-save the config data in case there's new info

                    if (!empty($mnc_multiview['configs'])) {
                        self::saveConfigs($mnc_multiview);
                    }
                    //and re-save the devices of the video wall itself in case the TVs were moved around / removed
                    self::saveDisplays($mnc_multiview);
                }

            }
        }

        //check if any existing devices are no longer found in the MNC data
        foreach ($all_multiviews as $angelicao_device) {
            $exists = false;
            $isDisconnected = false;
            foreach ($mnc_multiviews as $mnc_device) {
                if ($angelicao_device['mac'] === $angelicao_device['system_id'] . '_mv_' . intval($mnc_device['multiview_id']) &&//todo fake / temp field names
                    intval($angelicao_device['system_id']) === intval($mnc_device['system_id'])) {
                    $exists = true;

                    if (!empty($angelicao_device['isDisconnected'])) {
                        $isDisconnected = true;//will be used below to unmark it as disconnected if necessary
                    }
                }

            }

            if (empty($exists)) {
                $data = array(
                    'isDisconnected' => true,
                    'mac' => $angelicao_device['mac'],
                    'system_id' => $angelicao_device['system_id']
                );
                $sql = 'DELETE FROM devices WHERE system_id = ?:[system_id,i] AND mac = ?:[mac,s] LIMIT 1';
                DB::query($sql, $data);

            } else {//if the device is already in the data

                if ($isDisconnected === true) {//if the device was previously marked as disconnected, undo that.
                    $data = array(
                        'isDisconnected' => false,
                        'mac' => $angelicao_device['system_id'] . '_mv_' . $angelicao_device['multiview_id'],
                        'system_id' => $angelicao_device['system_id']
                    );
                    $sql = 'UPDATE devices SET isDisconnected = ?:[isDisconnected,b] WHERE system_id = ?:[system_id,i] AND mac = ?:[mac,s]';
                    DB::query($sql, $data);

                }
            }
        }

    }

    public static function saveConfigs($multiview) {
        foreach ($multiview['configs'] as $config) {
            $data = array(
                'mac_multiview' => $multiview['system_id'] . '_mv_' . $config['mac_multiview'],
                'config_id' => $config['config_id'],
                'name' => $config['name']
            );
            $sql = 'REPLACE INTO multiview_configs SET mac_multiview = ?:[mac_multiview,s], config_id = ?:[config_id,i], name = ?:[name,s]';
            DB::query($sql, $data);

            foreach ($config['windows'] as $window) {
                $window_data = array(
                    'mac_multiview' => $multiview['system_id'] . '_mv_' . $config['mac_multiview'],
                    'config_id' => $config['config_id'],
                    'window_id' => $window['window_id'],
                    'mac_tx' => $window['mac_tx'],
                    'x' => $window['x'],
                    'y' => $window['y'],
                    'width' => $window['width'],
                    'height' => $window['height'],
                    'audio_enabled' => $window['audio_enabled'],
                    'z_index' => $window['z_index'],
                );
                $sql = 'REPLACE INTO multiview_config_windows SET mac_multiview = ?:[mac_multiview,s], config_id = ?:[config_id,i], window_id = ?:[window_id,i], mac_tx = ?:[mac_tx,s], x = ?:[x,i], y = ?:[y,i], width = ?:[width,i], height = ?:[height,i], audio_enabled = ?:[audio_enabled,i], z_index = ?:[z_index,i]';
                DB::query($sql, $window_data);
            }
        }

        return;
    }

    public static function saveDisplays($multiview) {
        foreach ($multiview['displays'] as $display) {
            $display_data = array(
                'mac_rx' => $display['mac_rx'],
                'mac_multiview' => $multiview['system_id'] . '_mv_' . strtolower($multiview['multiview_id']),
                'x' => $display['x'],
                'y' => $display['y'],
                'width' => $display['width'],
                'height' => $display['height'],
                'audio_enabled' => $display['audio_enabled']
            );

            $sql = 'REPLACE INTO multiview_displays SET mac_rx = ?:[mac_rx,s], mac_multiview = ?:[mac_multiview,s], x = ?:[x,i], y = ?:[y,i], width = ?:[width,i], height = ?:[height,i], audio_enabled = ?:[audio_enabled,i]';
            DB::query($sql, $display_data);
        }

        return;
    }


    public static function addMultiview($multiview) {
        $sql = 'REPLACE INTO devices SET mac = ?:[mac,s], deleted = 0, system_id = ?:[system_id,i], customName = ?:[customName,s], icon = "", location_id = NULL, translate_x = NULL, translate_y = NULL, modelName = ?:[mac,s], isVideoWall = 1, videoWallType = "multi_view", videoWallSize = ?:[videoWallSize,s], isUsedInViewMode = 0';
        DB::query($sql, $multiview);

        if (!empty($multiview['configs'])) {
            self::saveConfigs($multiview);
        }

        if (!empty($multiview['displays'])) {
            self::saveDisplays($multiview);
        }
    }

    public static function updateMultiview($multiview) {
        //todo fake / temp field names
        $sql = 'UPDATE devices SET customName = ?:[customName,s], videoWallSize = ?:[videoWallSize,s] WHERE mac = ?:[mac,s]';
        DB::query($sql, $multiview);

        return;
    }

    public static function getMultiviews($include_deleted = false) {
        $sql = 'SELECT * FROM devices WHERE isVideoWall = 1 AND videoWallType = "multi_view" ';
        if (empty($include_deleted)) {
            $sql .= ' AND deleted != 1 ';
        }
        $multiviews = DB::query($sql, array());

        return $multiviews;
    }

    public static function getMultiview($mac) {
        if (empty($mac)) {
            return array(
                'status' => 'error',
                'message' => 'You must provide a multiview identifier',
                'code' => 'MV.100'
            );
        }
        $multiview = array();
        $user_account = MNC_API::getAccount();
        $data = array(
            'mac' => strtolower($mac),
            'user_id' => intval($user_account['user_id'])
        );


        $sql =
            'SELECT devices.*, multiview_configs.name as `connectedLayout`, multiview_configs.layout_id as `connectedLayoutId`, ' .
            '(SELECT COUNT(*) FROM multiview_currently_playing_connections WHERE mac_multiview = devices.mac) as `connectedLayoutChanges` FROM devices ' .
            'LEFT JOIN multiview_configs ON (devices.selected_layout = multiview_configs.config_id)';

        $sql .= ' WHERE LOWER(mac) = ?:[mac,s] AND isVideoWall != 0 AND mac NOT IN (select mac from blocked_devices WHERE user_id = ?:[user_id,i])';

        $_device = DB::query($sql, $data);
        if (!empty($_device)) {
            $_device = reset($_device);
            $multiview = $_device;

            $multiview['displays'] = self::getDisplays($multiview['mac']);

            $multiview['configs'] = self::getConfigs($multiview['mac']);

            //TODO add in the full wall as a config by default
            $multiview['configs'][] = array(
                'config_id' => 'all',
                'name' => 'All TVs',
                'windows' => array()
            );
            //end add in the full wall as a config by default

            //get specific connections made on the video wall (even if a layout was selected, user can still make arbitrary connections)
            $multiview['connectedLayoutSpecificConnections'] = self::getSpecificConnections($multiview['mac']);
        }

        return $multiview;
    }

    private static function getDisplays($mac) {
        if (empty($mac)) {
            return array(
                'status' => 'error',
                'message' => 'You must provide a multiview identifier',
                'code' => 'MV.100'
            );
        }

        $displays = DB::query('SELECT * FROM multiview_displays WHERE mac_multiview = ?:[mac_multiview,s]', array(
            'mac_multiview' => $mac
        ));

        return $displays;
    }

    private static function getConfigs($mac) {
        if (empty($mac)) {
            return array(
                'status' => 'error',
                'message' => 'You must provide a multiview identifier',
                'code' => 'MV.100'
            );
        }

        $configs = DB::query('SELECT * FROM multiview_configs WHERE mac_multiview = ?:[mac_multiview,s]', array('mac_multiview' => $mac));

        for ($i = 0; $i < sizeof($configs); $i++) {
            $configs['windows'] = DB::query('SELECT * FROM multiview_config_windows WHERE mac_multiview = ?:[mac_multiview,s] AND config_id = ?:[config_id,i]', array(
                'mac_multiview' => $mac,
                'config_id' => $configs[$i]['config_id']
            ));

        }

        return $configs;
    }

    private static function getSpecificConnections($mac) {
        if (empty($mac)) {
            return array(
                'status' => 'error',
                'message' => 'You must provide a multiview identifier',
                'code' => 'MV.100'
            );
        }
        return DB::query('SELECT * FROM multiview_currently_playing_connections WHERE mac_multiview = ?:[mac,s]', array('mac' => $mac));
    }

    public static function setSelectedConfig($mac, $config_id) {
        if (empty($mac) || empty($config_id)) {
            return array(
                'status' => 'error',
                'message' => 'you must specify a config and a multiview'
            );
        }

        $data = array(
            'mac' => $mac,
            'config_id' => $config_id,
        );
        $sql = 'UPDATE devices SET selected_layout = ?:[config_id,s], bypass_layout_source = "" WHERE mac = ?:[mac,s]';
        DB::query($sql, $data);

        //clear the specific connections if any. Meaning, the connections that were made on top of the layout selection
        DB::query('DELETE FROM multiview_currently_playing_connections WHERE mac_multiview = ?:[mac,s]', $data);

        return array(
            'status' => 'success'
        );
    }

    public static function setSpecificConnections($mac, $connections) {
        if (empty($mac)) {
            return array(
                'status' => 'error',
                'message' => 'You must provide a multiview identifier',
                'code' => 'MV.100'
            );
        }

        foreach ($connections as $connection) {

            $data = array(
                'source' => $connection['source'],
                'mac_multiview' => $mac,
                'window_id' => $connection['window_id']
            );

            $sql = 'REPLACE INTO multiview_currently_playing_connections SET mac_tx = ?:[source,s], mac_multiview = ?:[mac_multiview,s], window_id = ?:[window_id,i]';
            DB::query($sql, $data);
        }

        return array(
            'status' => 'success'
        );

    }
}