<?php
namespace MNC;

use AngelicaO\Devices;
use AngelicaO\User;

class MNC_API {

    private static $role_type = null;
    private static $user_id = null;
    private static $username = null;
    private static $password = null;
    private static $mncIP = '192.168.168.50';//localhost or 127.0.0.1 don't work, commands fail
    private static $mncPath = '/mnc/secure_api.php';
    private static $log_file = 'log/api_log.log';

    /**
     * Authenticate the credentials to the 811's internal API. Return the account type (admin, operator)
     * @param $username
     * @param $pass
     * @return bool|null|string
     */
    public static function getUserType($username, $pass) {

        return self::$role_type;
    }

    public static function setAccount($username, $pass) {

        //temporarily set the account to information passed in. We'll assume their login is correct for now, and confirm later
        self::$username = $username;
        self::$password = $pass;

        $all_users = User::getUsers(false);
        $user_exists = false;
        foreach ($all_users as $user) {
            //TODO check that the password is actually good
            if ($user['username'] == $username) {
                $user_exists = true;
                self::$user_id = $user['user_id'];
                self::$username = $username;
                self::$password = $pass;
                self::$role_type = $user['role_type'];
            }
        }

        if (empty($user_exists)) {
            self::$username = null;
            self::$password = null;
            return false;//TODO meaningful
        }

    }
    public static function getAccount() {
        return array(
            'user_id' => self::$user_id,
            'username' => self::$username,
            'role_type' => self::$role_type
        );
    }

    private static function make_api_call($command, $payload = false, $system_id = 0, $method = 'POST', $account = null) {

        self::$mncIP = $_SERVER['SERVER_ADDR'];
        if (self::$mncIP == '127.0.0.1') {
            self::$mncIP = exec('hostname -I');
        }
        $url = self::$mncIP . self::$mncPath;
        $curl = curl_init($url);

        $curl_data = array();

        if (empty($system_id)) {
            $system_id = 0;
        } else {
            $system_id = intval($system_id);
        }
        $curl_data['p_targetId'] = $system_id;

        if (empty($command)) {
            return null;//TODO meaningful
        } else {
            $curl_data['p_cmd'] = $command;
        }

        if (empty($account) && !empty(self::$username)) {//ie if you aren't logging in, then you need to have been logged in previously
            $curl_data['p_userName'] = self::$username;
            $curl_data['p_password'] = self::$password;
        } else if (!empty($account['username']) && !empty($account['password'])) {
            $curl_data['p_userName'] = $account['username'];
            $curl_data['p_password'] = $account['password'];
        } else {
            return null;//TODO meaningful
        }



        if (!empty($payload)) {
            $curl_data['p_data'] = $payload;
        }

        self::logRequest(array(
            'curl_data' => $curl_data,
            'url' => $url,
            'method' => $method
        ));

        if ($curl_data) {
            $curl_data = json_encode($curl_data, JSON_PRETTY_PRINT);

        }

        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);

        switch ($method) {
            case "POST":

                curl_setopt($curl, CURLOPT_POST, 1);
                if ($curl_data) {
                    curl_setopt($curl, CURLOPT_HTTPHEADER, array(
                        'Content-Type: application/json',
                        'Content-Length: ' . strlen($curl_data)
                    ));
                    curl_setopt($curl, CURLOPT_POSTFIELDS, $curl_data);

                }
                break;

            case "PUT":
                curl_setopt($curl, CURLOPT_PUT, 1);
                break;
            default:
                if ($curl_data) {
                    $url = sprintf("%s?%s", $url, http_build_query($curl_data));
                }
        }

        $result = curl_exec($curl);

        curl_close($curl);

        $response = json_decode($result, true);
        self::logResponse($response);

        return $response;
    }


    public static function getDevices() {
        $system_ids = self::getProductSystems();

        $detected_devices = array(
            'simpleDevices' => array(),
            'videoWalls' => array(),
            'multiViews' => array()
        );

        foreach ($system_ids as $system_id) {
            $simpleDevices = self::getSimpleDevicesForSystem($system_id);
            $videoWalls = self::getVideoWallsForSystem($system_id);
            //$multiViews = self::getMultiViewsForSystem($system_id);

            if (!empty($simpleDevices)) {
                $detected_devices['simpleDevices'] = array_merge($detected_devices['simpleDevices'], $simpleDevices);
            }

            if (!empty($videoWalls)) {
                $detected_devices['videoWalls'] = array_merge($detected_devices['videoWalls'], $videoWalls);
            }

            if (!empty($multiViews)) {
                $detected_devices['multiView'] = array_merge($detected_devices['multiView'], $multiViews);
            }
        }

        return $detected_devices;
    }

    private static function getProductSystems() {
        $result = self::make_api_call('getProductListSaved');
        $systems = array();

        if (!empty($result['p_rspStatus']) && strtoupper($result['p_rspStatus']) === 'SUCCESS') {
            for ($i = 0; $i < sizeof($result['p_data']); $i++) {
                if (!empty($result['p_data'][$i]['globalproductlistsavedID'])) {
                    $systems[] = $result['p_data'][$i]['globalproductlistsavedID'];
                }
            }
        }

        return $systems;
    }

    public static function getDevicesForSystem($system_id) {

    }

    private static function getSimpleDevicesForSystem($system_id) {
        $result = self::make_api_call('get_devices', array(), $system_id);
        $simpleDevices = array();

        if (!empty($result['p_data']) && is_array($result['p_data'])) {
            for ($i = 0; $i < sizeof($result['p_data']); $i++) {
                $device = $result['p_data'][$i];
                $device['system_id'] = $system_id;
                $device['isSimpleDevice'] = true;
                $simpleDevices[] = $device;
            }
        }

        return $simpleDevices;
    }

    private static function getVideoWallsForSystem($system_id) {
        $systemConfigs = self::make_api_call('vwGetConfigList', array(), $system_id);

        $videoWalls = array();
        $allConfigs = array();

        if (!empty($systemConfigs) && !empty($systemConfigs['p_rspStatus']) && strtoupper($systemConfigs['p_rspStatus']) === 'SUCCESS' && !empty($systemConfigs['p_data'] && is_array($systemConfigs['p_data']))) {
            for ($i = 0; $i < sizeof($systemConfigs['p_data']); $i++) {
                $systemConfig = $systemConfigs['p_data'][$i];
                $systemConfig['system_id'] = $system_id;
                $systemConfig['isVideoWall'] = true;


                $vwSize = explode('x', $systemConfig['VideoWallSize']);

                $vwSize = array(
                    'width' => intval($vwSize[0]),
                    'height' => intval($vwSize[1])
                );

                if (sizeof($systemConfig['devices']) === $vwSize['width'] * $vwSize['height']) {
                    $systemConfig['mac'] = $systemConfig['system_id'] . '_vw_' . $systemConfig['vwConfigId'];
                    $systemConfig['vwSize'] = $vwSize;
                    $systemConfig['configs'] = array();
                    $videoWalls[] = $systemConfig;
                } else {
                    $allConfigs[] = $systemConfig;
                }
            }

            //We know that a config belongs to a video wall if any of its devices' mac addresses match a mac address of the full VW
            for ($i = 0; $i < sizeof($allConfigs); $i++) {

                //pick the first device from the config and see if it matches any device from any video wall
                if (!empty($allConfigs[$i]['devices'][0])) {
                    $config_device_mac = $allConfigs[$i]['devices'][0]['macRx'];

                    for ($j = 0; $j < sizeof($videoWalls); $j++) {
                        $configBelongsToThisVideoWall = false;

                        for ($k = 0; $k < sizeof($videoWalls[$j]['devices']); $k++) {
                            if ($config_device_mac === $videoWalls[$j]['devices'][$k]['macRx']) {
                                $configBelongsToThisVideoWall = true;
                                break;
                            }
                        }

                        if ($configBelongsToThisVideoWall !== false) {
                            $videoWalls[$j]['configs'][] = $allConfigs[$i];
                            break;
                        }
                    }
                }

            }
        }

        return $videoWalls;
    }

    private static function getMultiViewsForSystem($system_id) {
        //TODO I don't know what the real command will actually look like!!! This is a temp/fake implementation
        $systemMultiviews = self::make_api_call('getMultiViews', array(), $system_id);

        $multiviews = array();

        if (!empty($systemMultiviews) && !empty($systemMultiviews['p_rspStatus']) && strtoupper($systemMultiviews['p_rspStatus']) === 'SUCCESS' && !empty($systemMultiviews['p_data'] && is_array($systemMultiviews['p_data']))) {
            for ($i = 0; $i < sizeof($systemMultiviews['p_data']); $i++) {
                $multiview = $systemMultiviews['p_data'][$i];
                $multiview['system_id'] = $system_id;
                $multiview['isVideoWall'] = true;
                $multiview['videoWallType'] = 'multi_view';
                $multiviews[] = $multiview;
            }
        }

        //TODO build the configs, displays, and config window data here

        /*
         * each config will look like this:
         * configs (mac_multiview, config_id, name, windows => array(window_id, mac_tx, x, y, with, height, audio_enabled, z_index))
         */

        //TODO
        /**
         * each display will look like this
         * displays (fields from DB go see there)
         */

        return $multiviews;
    }

    public static function getDevicesByMac($system_id, $macs) {
        $p_data = array();
        foreach ($macs as $mac) {
            $p_data[] = array(
                'mac' => $mac
            );
        }

        $devices = array();

        $response = self::make_api_call('get_devices', $p_data, $system_id);
        if (empty($response['p_data'])) {
            return array();
        }
        foreach ($response['p_data'] as $device) {
            if (!empty($device['mac'])) {
                $devices[$device['mac']] = $device;
            }
        }

        return $devices;
    }

    public static function getUsers() {

        $users = array();

        $mnc_users = self::make_api_call('getUsers', array(), 0);

        if (!empty($mnc_users) && !empty($mnc_users['p_rspStatus']) && strtoupper($mnc_users['p_rspStatus']) === 'SUCCESS' && !empty($mnc_users['p_data'] && is_array($mnc_users['p_data']))) {

            for ($i = 0; $i < sizeof($mnc_users['p_data']); $i++) {
                $users[] = $mnc_users['p_data'][$i];
            }
        }

        return $users;
    }

    private static function logRequest($request) {
        self::logData("------------------------\r\n\r\nREQUEST\r\n" . json_encode($request, JSON_PRETTY_PRINT) . "\r\n\r\n");
    }

    private static function logResponse($response) {
        self::logData("RESPONSE\r\n\r\n" . json_encode($response, JSON_PRETTY_PRINT) . "\r\n\r\n------------------------\r\n\r\n");
    }

    /**
     * log the request data to a log file
     * @param $content
     */
    private static function logData($content) {
        $path = __DIR__ . '/' . self::$log_file;

        if (file_exists($path)) {
            $filesize = filesize($path);
            if ($filesize / 1024 / 1024 > 4) {//4 megs
                exec('sudo rm /var/www/html/angelica-o/api/MNC/log/api_log.log');
                exec('sudo touch /var/www/html/angelica-o/api/MNC/log/api_log.log');
                exec('sudo chmod 777 /var/www/html/angelica-o/api/MNC/log/api_log.log');
            }
        }
        $fh = fopen($path, 'a+');

        fwrite($fh, $content . "\r\n");

        fclose($fh);
    }

}