<?php
use \Psr\Http\Message\ServerRequestInterface as Request;
use \Psr\Http\Message\ResponseInterface as Response;

$app->get('/devices', function (Request $request, Response $response) {
    \AngelicaO\AngelicaO::init();
    $devices = AngelicaO\Devices::getDevices();
    //var_dump($devices);exit(0);
    $response = $response->withJson($devices, 200);
    return $response;
});


$app->get('/devices/{mac}', function (Request $request, Response $response, $args) {
    \AngelicaO\AngelicaO::init();
    $result = AngelicaO\Devices::getDevice($args['mac']);

    if (!empty($result['status']) && $result['status'] === 'error') {
        $response = $response->withJson($result, 400);
    } else {
        $response = $response->withJson($result, 200);
    }

    return $response;
});

$app->post('/devices', function (Request $request, Response $response) {
    \AngelicaO\AngelicaO::init();
    $data = $request->getParsedBody();
    $device = AngelicaO\Devices::addDevice($data);

    if (!empty($device['status']) && $device['status'] === 'error') {
        $response = $response->withJson($device, 400);
    } else {
        $response = $response->withJson($device, 200);
    }

    return $response;
});

$app->post('/devices/{mac}/editIcon', function (Request $request, Response $response, $args) {
    \AngelicaO\AngelicaO::init();
    $data = $request->getParsedBody();

    $result = AngelicaO\Devices::editIcon($args['mac'], $data['icon']);

    if (!empty($result['status']) && $result['status'] === 'error') {
        $response = $response->withJson($result, 400);
    } else {
        $response = $response->withJson($result, 200);
    }

    return $response;
});

$app->delete('/devices/{mac}', function (Request $request, Response $response, $args) {
    \AngelicaO\AngelicaO::init();
    $data = $request->getParsedBody();
    $result = AngelicaO\Devices::deleteDevice($args['mac']);

    if (!empty($result['status']) && $result['status'] === 'error') {
        $response = $response->withJson($result, 400);
    } else {
        $response = $response->withJson($result, 200);
    }

    return $response;
});

$app->post('/devices/{mac}/updateName', function (Request $request, Response $response, $args) {
    \AngelicaO\AngelicaO::init();
    $data = $request->getParsedBody();

    $result = AngelicaO\Devices::updateName($args['mac'], $data['customName']);

    if (!empty($result['status']) && $result['status'] === 'error') {
        $response = $response->withJson($result, 400);
    } else {
        $response = $response->withJson($result, 200);
    }

    return $response;
});

$app->post('/devices/{mac}/setIsUsedInViewMode', function (Request $request, Response $response, $args) {
    \AngelicaO\AngelicaO::init();
    $data = $request->getParsedBody();

    $result = AngelicaO\Devices::setIsUsedInViewMode($args['mac'], $data['isUsedInViewMode']);

    if (!empty($result['status']) && $result['status'] === 'error') {
        $response = $response->withJson($result, 400);
    } else {
        $response = $response->withJson($result, 200);
    }

    return $response;
});

$app->post('/devices/{mac}/undeleteDevice', function (Request $request, Response $response, $args) {
    \AngelicaO\AngelicaO::init();
    $data = $request->getParsedBody();

    $result = AngelicaO\Devices::undeleteDevice($args['mac']);

    if (!empty($result['status']) && $result['status'] === 'error') {
        $response = $response->withJson($result, 400);
    } else {
        $response = $response->withJson($result, 200);
    }

    return $response;
});

$app->get('/devices/{display_mac}/sourcesAvailable', function (Request $request, Response $response, $args) {
    \AngelicaO\AngelicaO::init();

    $result = AngelicaO\Devices::getSourcesAvailableForDisplay($args['display_mac']);

    if (!empty($result['status']) && $result['status'] === 'error') {
        $response = $response->withJson($result, 400);
    } else {
        $response = $response->withJson($result, 200);
    }

    return $response;
});

$app->get('/devices/system/{systemId}/getSourcesInSystem', function (Request $request, Response $response, $args) {
    \AngelicaO\AngelicaO::init();

    $result = AngelicaO\Devices::getSourcesInSystem($args['systemId']);

    if (!empty($result['status']) && $result['status'] === 'error') {
        $response = $response->withJson($result, 400);
    } else {
        $response = $response->withJson($result, 200);
    }

    return $response;
});

$app->get('/devices/system/{systemId}/getDisplaysInSystem', function (Request $request, Response $response, $args) {
    \AngelicaO\AngelicaO::init();

    $result = AngelicaO\Devices::getDisplaysInSystem($args['systemId']);

    if (!empty($result['status']) && $result['status'] === 'error') {
        $response = $response->withJson($result, 400);
    } else {
        $response = $response->withJson($result, 200);
    }

    return $response;
});


/**
 * Removes a device from a its location - whatever the location is
 */
$app->post('/devices/{mac}/removeDeviceFromLocation', function (Request $request, Response $response, $args) {
    \AngelicaO\AngelicaO::init();

    $result = AngelicaO\Devices::removeDeviceFromLocation($args['mac']);

    if (!empty($result['status']) && $result['status'] === 'error') {
        $response = $response->withJson($result, 400);
    } else {
        $response = $response->withJson($result, 200);
    }

    return $response;
});

/**
 * Get list of users that are blocked from using a particular device
 */

$app->get('/devices/getBlockedUsers/{mac}', function (Request $request, Response $response, $args) {
    \AngelicaO\AngelicaO::init();

    $result = AngelicaO\Devices::getBlockedUsersForDevice($args['mac']);

    if (!empty($result['status']) && $result['status'] === 'error') {
        $response = $response->withJson($result, 400);
    } else {
        $response = $response->withJson($result, 200);
    }

    return $response;
});

/**
 * Blocks some users from using a particular device
 */
$app->post('/devices/{mac}/blockUsers', function (Request $request, Response $response, $args) {
    \AngelicaO\AngelicaO::init();
    $data = $request->getParsedBody();

    $result = AngelicaO\Devices::blockUsers($args['mac'], $data['users']);

    if (!empty($result['status']) && $result['status'] === 'error') {
        $response = $response->withJson($result, 400);
    } else {
        $response = $response->withJson($result, 200);
    }

    return $response;
});