<?php
use \Psr\Http\Message\ServerRequestInterface as Request;
use \Psr\Http\Message\ResponseInterface as Response;


$app->get('/devices/getVideoWall/{mac}', function (Request $request, Response $response, $args) {
    \AngelicaO\AngelicaO::init();

    $result = AngelicaO\VideoWall::getVideoWall($args['mac'], null);

    if (!empty($result['status']) && $result['status'] === 'error') {
        $response = $response->withJson($result, 400);
    } else {
        $response = $response->withJson($result, 200);
    }

    return $response;
});

$app->post('/devices/{videoWallFullConfigId}/saveVideoWallLayout', function (Request $request, Response $response, $args) {
    \AngelicaO\AngelicaO::init();
    $data = $request->getParsedBody();

    $result = AngelicaO\VideoWall::saveVideoWallLayout($data['layout']);

    if (!empty($result['status']) && $result['status'] === 'error') {
        $response = $response->withJson($result, 400);
    } else {
        $response = $response->withJson($result, 200);
    }

    return $response;
});

$app->post('/devices/{videoWallFullConfigId}/setSelectedLayout', function (Request $request, Response $response, $args) {
    \AngelicaO\AngelicaO::init();
    $data = $request->getParsedBody();

    $result = AngelicaO\VideoWall::setSelectedLayout($data['system_id'], $args['videoWallFullConfigId'], $data['layout_id']);

    if (!empty($result['status']) && $result['status'] === 'error') {
        $response = $response->withJson($result, 400);
    } else {
        $response = $response->withJson($result, 200);
    }

    return $response;
});

/**
 * Used to store specific connections made in a video wall after a layout is selected. This is used to show the user what's currently
 * playing when he opens the video wall page. The 811 can give us the connectedMac for each device, but we'd be required to guess which is the active layout,
 * which isn't necessarily possible given that specific connections can be made after a layout is selected
 */
$app->post('/devices/{videoWallFullConfigId}/setVideoWallSpecificConnections', function (Request $request, Response $response, $args) {
    \AngelicaO\AngelicaO::init();
    $data = $request->getParsedBody();

    $result = AngelicaO\VideoWall::setVideoWallSpecificConnections($data['system_id'], $args['videoWallFullConfigId'], (!empty($data['connections'])) ? $data['connections'] : array());

    if (!empty($result['status']) && $result['status'] === 'error') {
        $response = $response->withJson($result, 400);
    } else {
        $response = $response->withJson($result, 200);
    }

    return $response;
});

/**
 * Deletes a layout and clears the connectedLayout data optionally
 */
$app->delete('/devices/layout/{layout_id}', function (Request $request, Response $response, $args) {
    \AngelicaO\AngelicaO::init();
    $data = $request->getParsedBody();
    $result = AngelicaO\VideoWall::deleteLayout($args['layout_id'], $data);

    if (!empty($result['status']) && $result['status'] === 'error') {
        $response = $response->withJson($result, 400);
    } else {
        $response = $response->withJson($result, 200);
    }

    return $response;
});

/**
 * Allows you to save the source applied to an entire video wall, when a layout wasn't used (Ex. in visual mode; or in
 * video wall page when no layout is selected).
 */
$app->post('/devices/{videoWallFullConfigId}/setFullVideoWallSource', function (Request $request, Response $response, $args) {
    \AngelicaO\AngelicaO::init();
    $data = $request->getParsedBody();

    $result = AngelicaO\VideoWall::setFullVideoWallSource($data['system_id'], $args['videoWallFullConfigId'], $data['source_mac']);

    if (!empty($result['status']) && $result['status'] === 'error') {
        $response = $response->withJson($result, 400);
    } else {
        $response = $response->withJson($result, 200);
    }

    return $response;
});